#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "DnsMXRecordForm.h"

#include <openwbem/OW_CIMProperty.hpp>
#include <openwbem/OW_CIMValue.hpp>
#include <openwbem/OW_CIMQualifier.hpp>
#include <openwbem/OW_CIMException.hpp>
#include <openwbem/OW_Bool.hpp>

using namespace Gtk;
using namespace OpenWBEM;

DnsMXRecordForm::DnsMXRecordForm()
	: table(3, 2),
	name_label("Domain Name"),
	server_label("Mail server"),
	priority_label("Mail server priority")
{
	// set table parameters
	table.set_row_spacings(8);
	table.set_col_spacings(10);
	table.set_border_width(8);

	// set label properties
	name_label.set_alignment(ALIGN_LEFT, ALIGN_CENTER);
	name_label.show();
	server_label.set_alignment(ALIGN_LEFT, ALIGN_CENTER);
	server_label.show();
	priority_label.set_alignment(ALIGN_LEFT, ALIGN_CENTER);
	priority_label.show();
	
	// attach labels to the table
	table.attach(name_label, 0, 1, 0, 1, FILL, SHRINK);
	table.attach(server_label, 0, 1, 1, 2, FILL, SHRINK);
	table.attach(priority_label, 0, 1, 2, 3, FILL, SHRINK);

	// set entry properties
	name_entry.set_editable(false);
	name_entry.show();
	server_entry.show();
	priority_entry.set_increments(1.0, 10.0);
	priority_entry.set_range(0.0, double(2 << 15));
	priority_entry.show();

	// attach entries to the table
	table.attach(name_entry, 1, 2, 0, 1,
			FILL | EXPAND, SHRINK);
	table.attach(server_entry, 1, 2, 1, 2,
			FILL | EXPAND, SHRINK);
	table.attach(priority_entry, 1, 2, 2, 3,
			FILL | EXPAND, SHRINK);
}

void
DnsMXRecordForm::set_instance(const CIMObjectPath &cop,
		Reference<CIMInstance> inst)
{
	this->inst = inst;

	// fill in widgets with values from instance
	update();
}

Gtk::Widget *
DnsMXRecordForm::getWidget()
{
	return &table;
}

void
DnsMXRecordForm::update()
{
	OW_String string;
	OW_Bool boolean;
	
	// host name
	CIMValue value = inst->getProperty("OwnerName").getValue();
	if (value)
		value.get(string);
	else
		string = "";
	name_entry.set_text(string.c_str());
	
	// mail server
	value = inst->getProperty("MailExchange").getValue();
	if (value)
		value.get(string);
	else
		string = "";
	server_entry.set_text(string.c_str());

	// mail server priority
	value = inst->getProperty("Preference").getValue();
	if (value)
	{
		UInt16 i;
		value.get(i);
		priority_entry.set_value(double(i));
	}
	else
	{
		priority_entry.set_text("");
	}
}

void
DnsMXRecordForm::applyChanges()
{
	// mail server
	inst->setProperty("MailExchange",
			CIMValue(server_entry.get_text().c_str()));
	
	// mail server priority
	UInt16 pref = priority_entry.get_value_as_int();
	inst->setProperty("Preference", CIMValue(pref));
}
