#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "DnsSOARecordForm.h"

#include <openwbem/OW_CIMProperty.hpp>
#include <openwbem/OW_CIMValue.hpp>
#include <openwbem/OW_CIMQualifier.hpp>
#include <openwbem/OW_CIMException.hpp>
#include <openwbem/OW_Bool.hpp>

using namespace Gtk;
using namespace OpenWBEM;

DnsSOARecordForm::DnsSOARecordForm(CIMInstanceHandle objectref)
	: SimpleCIMForm(objectref),
	table(7, 2),
	intervals_table(5, 2),
	serial_increment_btn("Increment")
{
	// set table parameters
	table.set_col_spacings(10);
	table.set_border_width(8);
	
	// create bound entries
	serial_bound_entry = new_entry("SerialNumber");
	master_bound_entry = new_entry("PrimaryServer");
	responsible_person_bound_entry = new_entry("ResponsibleParty");
	refresh_bound_entry = new_entry("RefreshInterval");
	retry_bound_entry = new_entry("RetryDelay");
	expire_bound_entry = new_entry("ExpireLimit");
	negcache_bound_entry = new_entry("NegCache");
	ttl_bound_entry = new_entry("TTL");

	// attach labels to the table
	table.attach(*serial_bound_entry->get_label(), 0, 2, 0, 1, FILL, SHRINK);
	table.set_row_spacing(1, 8);
	table.attach(*master_bound_entry->get_label(), 0, 2, 2, 3, FILL, SHRINK);
	table.set_row_spacing(3, 8);
	table.attach(*responsible_person_bound_entry->get_label(), 0, 2, 4, 5,
			FILL, SHRINK);
	table.set_row_spacing(5, 12);

	// attach entries to the table
	table.attach(*serial_bound_entry->get_entry(), 0, 1, 1, 2,
			FILL | EXPAND, SHRINK);
	table.attach(*master_bound_entry->get_entry(), 0, 1, 3, 4,
			FILL | EXPAND, SHRINK);
	table.attach(*responsible_person_bound_entry->get_entry(), 0, 1, 5, 6,
			FILL | EXPAND, SHRINK);

	// attach button
	serial_increment_btn.show();
	table.attach(serial_increment_btn, 1, 2, 1, 2, FILL, SHRINK);

	// intervals table
	intervals_table.attach(*refresh_bound_entry->get_label(), 0, 1, 0, 1,
			FILL, SHRINK);
	intervals_table.attach(*retry_bound_entry->get_label(), 0, 1, 1, 2,
			FILL, SHRINK);
	intervals_table.attach(*expire_bound_entry->get_label(), 0, 1, 2, 3,
			FILL, SHRINK);
	intervals_table.attach(*negcache_bound_entry->get_label(), 0, 1, 3, 4,
			FILL, SHRINK);
	intervals_table.attach(*ttl_bound_entry->get_label(), 0, 1, 4, 5,
			FILL, SHRINK);

	intervals_table.attach(*refresh_bound_entry->get_entry(), 1, 2, 0, 1,
			FILL | EXPAND, SHRINK);
	intervals_table.attach(*retry_bound_entry->get_entry(), 1, 2, 1, 2,
			FILL | EXPAND, SHRINK);
	intervals_table.attach(*expire_bound_entry->get_entry(), 1, 2, 2, 3,
			FILL | EXPAND, SHRINK);
	intervals_table.attach(*negcache_bound_entry->get_entry(), 1, 2, 3, 4,
			FILL | EXPAND, SHRINK);
	intervals_table.attach(*ttl_bound_entry->get_entry(), 1, 2, 4, 5,
			FILL | EXPAND, SHRINK);

	intervals_table.set_row_spacings(8);
	intervals_table.set_col_spacings(10);
	intervals_table.show();
	table.attach(intervals_table, 0, 1, 6, 7,
			FILL | EXPAND, FILL | EXPAND);
}

Gtk::Widget *
DnsSOARecordForm::getWidget()
{
	return &table;
}
