#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "NagiosServiceList.h"
#include "CIMOMHandle.h"
#include "NagiosWindow.h"
#include "NagiosHostDialog.h"
#include "ModifyInstanceDialog.h"
#include "ClassHandlerRegistry.h"
#include "InstancePropertyListForm.h"

#include <openwbem/OW_Enumeration.hpp>
#include <openwbem/OW_CIMException.hpp>
#include <openwbem/OW_CIMProperty.hpp>
#include <openwbem/OW_CIMValue.hpp>
#include <gtkmm/cellrenderertext.h>

using namespace Gtk;
using namespace OpenWBEM;

NagiosServiceList::NagiosServiceList(bool show_host_column)
{
	m_store = ListStore::create(Columns);
	set_model(m_store);

	if (show_host_column)
	{
		append_column("Host", Columns.host);
	}
	append_column("Service", Columns.service);

	CellRendererText *renderer = manage(new CellRendererText());
	int column = append_column("Status", *renderer) - 1;
	get_column(column)->add_attribute(renderer->property_text(),
			Columns.status);
	get_column(column)->add_attribute(renderer->property_background(),
			Columns.status_color);

	append_column("Last Check", Columns.last_check);
	append_column("Duration", Columns.duration);
	append_column("Attempt", Columns.attempt);
	append_column("Status Information", Columns.information);
}

void
NagiosServiceList::show_host(CIMObjectPath host_cop)
{
	m_store->clear();

	try
	{
		CIMInstanceEnumeration result;
		result = CIMOMHandle::associatorsE(host_cop,
				"Config4GNU_NagiosHostService");
	
		while (result.hasMoreElements())
		{
			CIMInstance inst = result.nextElement();
			add_service(inst);
		}
	}
	catch (CIMException e)
	{
		g_warning("%s", e.getMessage());
	}
}

void
NagiosServiceList::show_hostgroup(CIMObjectPath hostgroup_cop)
{
	m_store->clear();

	try
	{
		CIMInstanceEnumeration result;
		result = CIMOMHandle::associatorsE(hostgroup_cop);
	
		while (result.hasMoreElements())
		{
			CIMInstance inst = result.nextElement();
			add_service(inst);
		}
	}
	catch (CIMException e)
	{
		g_warning("%s", e.getMessage());
	}
}

void
NagiosServiceList::add_service(CIMInstance &inst)
{
	TreeModel::iterator iter = m_store->append();
	TreeModel::Row row = *iter;
			
	String string;
	inst.getProperty("HostAlias").getValue().get(string);
	row[Columns.host] = string.c_str();

	inst.getProperty("ServiceDescription").getValue().get(string);
	row[Columns.service] = string.c_str();

	inst.getProperty("Status").getValue().get(string);
	row[Columns.status] = string.c_str();
	row[Columns.status_color] = (string == "OK" ? "green" :
			string == "WARNING" ? "yellow" :
			string == "CRITICAL" ? "red" : "gray");

	CIMDateTime x;
	inst.getProperty("LastCheckTime").getValue().get(x);
	time_t lastCheckTime = convertTime(x);
	char timebuf[200];
	strftime(timebuf, sizeof(timebuf), "%x %X",
			localtime(&lastCheckTime));
	row[Columns.last_check] = timebuf;

	inst.getProperty("LastStateChange").getValue().get(x);
	time_t lastStateChange = convertTime(x);
	row[Columns.duration] = duration2string(time(NULL)
			- lastStateChange);

	unsigned int retryNumber, maxRetries;
	inst.getProperty("RetryNumber").getValue().get(retryNumber);
	inst.getProperty("MaxRetries").getValue().get(maxRetries);
	row[Columns.attempt] =
		(String(retryNumber) + "/" + String(maxRetries)).c_str();

	inst.getProperty("PluginOutput").getValue().get(string);
	row[Columns.information] = string.c_str();
			
	row[Columns.instobj] = inst;
}

Glib::ustring
NagiosServiceList::duration2string(time_t duration)
{
	int seconds = duration % 60;
	duration /= 60;
	int minutes = duration % 60;
	duration /= 60;
	int hours = duration % 24;
	int days = duration / 24;

	char *tmp = g_strdup_printf("%dd %dh %dm %ds",
			days, hours, minutes, seconds);
	Glib::ustring result(tmp);
	g_free(tmp);
	return result;
}

void
NagiosServiceList::on_row_activated(const TreeModel::Path &path,
		TreeViewColumn *column)
{
	TreeModel::iterator iter = m_store->get_iter(path);
	TreeModel::Row row = *iter;
	
	CIMInstance inst = row[Columns.instobj];
	on_instance_activated(inst);
}

void
NagiosServiceList::on_instance_activated(CIMInstance inst)
{
	CIMObjectPath cop(CIMOMHandle::getDefaultNamespace(), inst);

	ClassHandler *handler = ClassHandlerRegistry::get_handler(
			inst.getClassName());
	if (handler != NULL)
	{
		handler->do_properties(cop, (Window *) get_toplevel());
	}
	else
	{
		g_message("classname = %s",
			inst.getClassName().c_str());
		ModifyInstanceDialog *dlg = new ModifyInstanceDialog(cop);
		dlg->add_form(new InstancePropertyListForm, "Advanced");
		dlg->show();
	}
}
