#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "StatusMapWindow.h"
#include "CIMOMHandle.h"
#include "SelectHostDialog.h"
#include "NagiosHostDialog.h"

#include <gtkmm.h>
#include <libgnomecanvasmm/rect.h>
#include <libgnomecanvasmm/ellipse.h>
#include <libgnomecanvasmm/text.h>

#include <openwbem/OW_CIMProtocolIFC.hpp>
#include <openwbem/OW_HTTPClient.hpp>
#include <openwbem/OW_CIMXMLCIMOMHandle.hpp>
#include <openwbem/OW_CIMException.hpp>
#include <openwbem/OW_CIMProperty.hpp>
#include <openwbem/OW_CIMValue.hpp>

using namespace Gtk;
using namespace OpenWBEM;
using namespace Gnome::Canvas;

StatusMapWindow::StatusMapWindow(const CIMInstanceHandle &handle)
	: canvas(handle)
{
	this->handle = handle;

	set_title("Nagios Status Map");
	set_default_size(600, 450);
	
	// Map menu:
	{
		Menu::MenuList &menulist = menu_map.items();
		
		menulist.push_back(Menu_Helpers::MenuElem("_Add Host...",
					SigC::slot(*this, &StatusMapWindow::on_menu_map_add_host)));
		menulist.push_back(Menu_Helpers::StockMenuElem(Stock::CLOSE,
					SigC::slot(*this, &StatusMapWindow::on_menu_map_close)));
	}
	
	// View menu:
	{
		Menu::MenuList &menulist = menu_view.items();
		
		menulist.push_back(Menu_Helpers::MenuElem("_Find Class...",
				SigC::slot(*this, &StatusMapWindow::on_menu_schema_find_class)));
	}

	// Help menu:
	{
		Menu::MenuList &menulist = menu_help.items();
		
		menulist.push_back(Menu_Helpers::MenuElem("_About Provider...",
				SigC::slot(*this, &StatusMapWindow::on_menu_help_about)));
	}
	
	menu_bar.items().push_back(Menu_Helpers::MenuElem("_Map", menu_map));
	menu_bar.items().push_back(Menu_Helpers::MenuElem("_View", menu_view));
	menu_bar.items().push_back(Menu_Helpers::MenuElem("_Help", menu_help));
	
	vbox.pack_start(menu_bar, PACK_SHRINK);
	
	canvas.show();
	canvas_scrolled.add(canvas);

	canvas_scrolled.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC);
	canvas_scrolled.set_shadow_type(SHADOW_IN);
	vbox.pack_start(canvas_scrolled);
	
	vbox.show();
	add(vbox);
	show_all_children();
}

void
StatusMapWindow::on_menu_map_add_host()
{
	SelectHostDialog dlg;
	dlg.set_transient_for(*this);
	int response = dlg.run();
	if (response = RESPONSE_OK)
	{
		CIMInstanceHandle selected_host = dlg.get_selection();
		canvas.add_host(selected_host, 50.0, 50.0);
	}
}

void
StatusMapWindow::on_menu_map_close()
{
	hide();
}

void
StatusMapWindow::on_menu_schema_find_class()
{
}

void
StatusMapWindow::on_menu_help_about()
{
}
