#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "SambaGlobalSettingsGeneralForm.h"

#include <openwbem/OW_CIMValue.hpp>
#include <openwbem/OW_CIMQualifier.hpp>
#include <openwbem/OW_CIMException.hpp>
#include <openwbem/OW_Bool.hpp>
#include <iostream>

using namespace Gtk;
using namespace OpenWBEM;

SambaGlobalSettingsGeneralForm::SambaGlobalSettingsGeneralForm()
	: table(4, 2),
	server_name_label("Server Name"),
	server_description_label("Server Description"),
	workgroup_label("Workgroup"),
	master_browser_label("Master Browser"),
	master_browser_yes(master_browser_group, "Yes"),
	master_browser_no(master_browser_group, "No")
{
	// set table parameters
	table.set_row_spacings(8);
	table.set_col_spacings(10);
	table.set_border_width(8);
	
	// set label alignments
	server_name_label.set_alignment(ALIGN_LEFT, ALIGN_CENTER);
	server_description_label.set_alignment(ALIGN_LEFT, ALIGN_CENTER);
	workgroup_label.set_alignment(ALIGN_LEFT, ALIGN_CENTER);
	master_browser_label.set_alignment(ALIGN_LEFT, ALIGN_CENTER);
	
	// attach labels to the table
	table.attach(server_name_label, 0, 1, 0, 1, FILL, SHRINK);
	table.attach(server_description_label, 0, 1, 1, 2, FILL, SHRINK);
	table.attach(workgroup_label, 0, 1, 2, 3, FILL, SHRINK);
	table.attach(master_browser_label, 0, 1, 3, 4, FILL, SHRINK);

	// attach entries to the table
	table.attach(server_name_entry, 1, 2, 0, 1,
			FILL | EXPAND, SHRINK);
	table.attach(server_description_entry, 1, 2, 1, 2,
			FILL | EXPAND, SHRINK);
	table.attach(workgroup_entry, 1, 2, 2, 3,
			FILL | EXPAND, SHRINK);
	
	// add radio buttons to access hbox, and add vbox to table
	master_browser_box.pack_start(master_browser_yes);
	master_browser_box.pack_start(master_browser_no);
	table.attach(master_browser_box, 1, 2, 3, 4,
			FILL | EXPAND, SHRINK);
			
	// TODO... make all child widgets visible
}

void
SambaGlobalSettingsGeneralForm::set_instance(const CIMObjectPath &cop,
		Reference<CIMInstance> inst)
{
	this->inst = inst;

	// fill in widgets with values from instance
	update();
}

Gtk::Widget *
SambaGlobalSettingsGeneralForm::getWidget()
{
	return &table;
}

void
SambaGlobalSettingsGeneralForm::update()
{
	String string;
	Bool boolean;
	
	// server name
	CIMValue value = inst->getProperty("NetBIOSName").getValue();
	if (value)
		value.get(string);
	else
		string = "";
	server_name_entry.set_text(string.c_str());
	
	// server description
	value = inst->getProperty("ServerString").getValue();
	if (value)
		value.get(string);
	else
		string = "";
	server_description_entry.set_text(string.c_str());
	
	// workgroup
	value = inst->getProperty("Workgroup").getValue();
	if (value)
		value.get(string);
	else
		string = "";
	workgroup_entry.set_text(string.c_str());

	// master browser
	value = inst->getProperty("LocalMaster").getValue();
	if (value)
		value.get(boolean);
	else
		boolean = false;
	master_browser_yes.set_active(boolean);
	master_browser_no.set_active(!boolean);
}

void
SambaGlobalSettingsGeneralForm::applyChanges()
{
	// server name
	// if user specified empty string, make it NULL rather than an empty string
	String server_name_text(server_name_entry.get_text().c_str());
	inst->setProperty("NetBIOSName", server_name_text == "" ?
			CIMValue(CIMNULL) : CIMValue(server_name_text));

	// server string
	inst->setProperty("ServerString",
			CIMValue(server_description_entry.get_text().c_str()));
		
	// workgroup
	inst->setProperty("Workgroup",
			CIMValue(workgroup_entry.get_text().c_str()));
		
	// master browser
	inst->setProperty("LocalMaster",
			CIMValue(OW_Bool(master_browser_yes.get_active())));
}
