#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "SambaNewShareDialog.h"
#include "CIMOMHandle.h"

#include <gtkmm/messagedialog.h>
#include <gtkmm/stock.h>

#include <openwbem/OW_CIMProperty.hpp>
#include <openwbem/OW_CIMValue.hpp>
#include <openwbem/OW_CIMClass.hpp>
#include <iostream>

using namespace Gtk;
using namespace OpenWBEM;

SambaNewShareDialog::SambaNewShareDialog(CIMObjectPath samba_service_cop)
	: table(4, 2),
	computer_label("Computer:"),
	path_label("Folder to share:"),
	name_label("Share name:"),
	description_label("Share description:")
{
	// initialize my properties
	this->samba_service_cop = samba_service_cop;

	set_title("Create Shared Folder");

	// initialize labels and entry properties
	computer_label.set_alignment(ALIGN_LEFT, ALIGN_CENTER);
	path_label.set_alignment(ALIGN_LEFT, ALIGN_CENTER);
	name_label.set_alignment(ALIGN_LEFT, ALIGN_CENTER);
	description_label.set_alignment(ALIGN_LEFT, ALIGN_CENTER);
	computer_name.set_alignment(ALIGN_LEFT, ALIGN_CENTER);
	path_entry.set_activates_default(true);
	name_entry.set_activates_default(true);
	description_entry.set_activates_default(true);	

	// add widget table to this dialog
	table.set_row_spacings(8);
	table.set_col_spacings(10);
	table.set_border_width(8);
	get_vbox()->pack_start(table);

	// add labels and entries to table
	table.attach(computer_label, 0, 1, 0, 1, FILL);
	table.attach(path_label, 0, 1, 1, 2, FILL);
	table.attach(name_label, 0, 1, 2, 3, FILL);
	table.attach(description_label, 0, 1, 3, 4, FILL);

	table.attach(computer_name, 1, 2, 0, 1);
	table.attach(path_entry, 1, 2, 1, 2);
	table.attach(name_entry, 1, 2, 2, 3);
	table.attach(description_entry, 1, 2, 3, 4);

	// initialize computer name label to name of system according to the
	// samba_service CIMObjectPath
	String system_name;
	samba_service_cop.getKey("SystemName").getValue().get(system_name);
	computer_name.set_text(system_name.c_str());
	
	// add buttons
	add_button(Stock::CANCEL, RESPONSE_CANCEL);
	add_button(Stock::OK, RESPONSE_OK);
	set_default_response(RESPONSE_OK);
	
	// make sure all children widgets are visible
	show_all_children();
}

CIMInstance
SambaNewShareDialog::createInstanceObject()
{
	CIMClass cimClass(CIMOMHandle::getClass("Config4GNU_SambaShare"));
	CIMInstance inst = cimClass.newInstance();
	
	// key properties
	inst.setProperty("SystemName",
		samba_service_cop.getKey("SystemName").getValue());
	inst.setProperty("SystemCreationClassName",
		samba_service_cop.getKey("SystemCreationClassName").getValue());
	inst.setProperty("Name", CIMValue(name_entry.get_text().c_str()));
	inst.setProperty("CreationClassName", CIMValue("Config4GNU_SambaShare"));
	
	// other properties
	inst.setProperty("Path", CIMValue(path_entry.get_text().c_str()));
	inst.setProperty("Comment", CIMValue(description_entry.get_text().c_str()));
	
	return inst;
}
