#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "SambaShareGeneralForm.h"

#include <gtkmm/messagedialog.h>
#include <gtkmm/stock.h>

#include <openwbem/OW_CIMValue.hpp>
#include <openwbem/OW_CIMQualifier.hpp>
#include <openwbem/OW_CIMException.hpp>
#include <openwbem/OW_Bool.hpp>
#include <iostream>

using namespace Gtk;
using namespace OpenWBEM;

SambaShareGeneralForm::SambaShareGeneralForm()
	: general_table(4, 2),
	general_share_name_label("Share Name"),
	general_path_label("Path"),
	general_comment_label("Comment"),
	general_access_label("Access"),
	general_access_readonly(general_access_group, "Read Only"),
	general_access_writeable(general_access_group, "Writable")
{
	// set table parameters
	general_table.set_row_spacings(8);
	general_table.set_col_spacings(10);
	general_table.set_border_width(8);
	
	// set label alignments
	general_share_name_label.set_alignment(ALIGN_LEFT, ALIGN_CENTER);
	general_path_label.set_alignment(ALIGN_LEFT, ALIGN_CENTER);
	general_comment_label.set_alignment(ALIGN_LEFT, ALIGN_CENTER);
	general_access_label.set_alignment(ALIGN_LEFT, ALIGN_CENTER);
	
	// attach labels to the table
	general_table.attach(general_share_name_label, 0, 1, 0, 1, FILL, SHRINK);
	general_table.attach(general_path_label, 0, 1, 1, 2, FILL, SHRINK);
	general_table.attach(general_comment_label, 0, 1, 2, 3, FILL, SHRINK);
	general_table.attach(general_access_label, 0, 1, 3, 4, FILL, SHRINK);

	// attach entries to the table
	general_table.attach(general_share_name_entry, 1, 2, 0, 1,
			FILL | EXPAND, SHRINK);
	general_table.attach(general_path_entry, 1, 2, 1, 2,
			FILL | EXPAND, SHRINK);
	general_table.attach(general_comment_entry, 1, 2, 2, 3,
			FILL | EXPAND, SHRINK);
	
	// add radio buttons to access hbox, and add vbox to table
	general_access_box.pack_start(general_access_readonly);
	general_access_box.pack_start(general_access_writeable);
	general_table.attach(general_access_box, 1, 2, 3, 4,
			FILL | EXPAND, SHRINK);
			
	// TODO... make all child widgets visible
}

void
SambaShareGeneralForm::set_instance(const CIMObjectPath &cop,
		Reference<CIMInstance> inst)
{
	this->inst = inst;

	// fill in widgets with values from instance
	update();
}

Gtk::Widget *
SambaShareGeneralForm::getWidget()
{
	return &general_table;
}

void
SambaShareGeneralForm::update()
{
	String string;
	Bool boolean;
	
	// share name
	CIMValue value = inst->getProperty("Name").getValue();
	if (value)
		value.get(string);
	else
		string = "";
	general_share_name_entry.set_text(string.c_str());
	
	// path
	value = inst->getProperty("Path").getValue();
	if (value)
		value.get(string);
	else
		string = "";
	general_path_entry.set_text(string.c_str());
	
	// comment
	value = inst->getProperty("Description").getValue();
	if (value)
		value.get(string);
	else
		string = "";
	general_comment_entry.set_text(string.c_str());

	// access
	value = inst->getProperty("ReadOnly").getValue();
	if (value)
		value.get(boolean);
	else
		boolean = false;
	general_access_readonly.set_active(boolean);
	general_access_writeable.set_active(!boolean);
}

void
SambaShareGeneralForm::applyChanges()
{
	// path
	inst->setProperty("Path",
			CIMValue(general_path_entry.get_text().c_str()));
		
	// comment
	inst->setProperty("Description",
			CIMValue(general_comment_entry.get_text().c_str()));
		
	// access
	inst->setProperty("ReadOnly",
			CIMValue(OW_Bool(general_access_readonly.get_active())));
}
