// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "AddCIMObjectDialog.h"
#include "CIMOMHandle.h"
#include <gtkmm/stock.h>
#include <openwbem/OW_CIMClass.hpp>
#include <openwbem/OW_CIMValue.hpp>
#include <assert.h>

using namespace Gtk;
using namespace Glib;
using namespace OpenWBEM;

AddCIMObjectDialog::AddCIMObjectDialog()
	: class_label("CIM class:"),
	  keys_label("Key values:")
{
	set_title("Add CIM Object");
	set_default_size(250, 250);
	get_vbox()->set_border_width(6);

	class_label.show();
	class_hbox.pack_start(class_label, PACK_SHRINK);
	
	class_entry.signal_focus().connect_notify(
			SigC::slot(*this, &AddCIMObjectDialog::on_entry_focus));
	class_entry.show();
	class_hbox.pack_start(class_entry, PACK_EXPAND_WIDGET);

	class_hbox.show();
	get_vbox()->pack_start(class_hbox, PACK_SHRINK);
	
	keys_label.set_alignment(ALIGN_LEFT, ALIGN_CENTER);
	keys_label.show();
	get_vbox()->pack_start(keys_label, PACK_SHRINK);
	
	// prepare TreeView widget that displays the keys list
	keys_view.append_column("Key", KeysColumns.name);
	CellRendererText *value_renderer = manage(new CellRendererText());
	value_renderer->property_editable() = true;
	value_renderer->signal_edited().connect(
			SigC::slot(*this, &AddCIMObjectDialog::on_value_edited));
	keys_view.append_column("Value", *value_renderer);
	keys_view.get_column(1)->add_attribute(
			value_renderer->property_text(), KeysColumns.value);
	keys_view.show();
	keys_scrolled.add(keys_view);
	
	// prepare keys list scrolled window
	keys_scrolled.set_shadow_type(SHADOW_IN);
	keys_scrolled.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC);
	keys_scrolled.show();
	get_vbox()->pack_start(keys_scrolled, PACK_EXPAND_WIDGET);

	// add buttons
	add_button(Stock::CANCEL, RESPONSE_CANCEL);
	add_button(Stock::OK, RESPONSE_OK);
}

void
AddCIMObjectDialog::on_entry_focus(DirectionType direction)
{
	update_keys();
}

void
AddCIMObjectDialog::update_keys()
{
	String new_class_name(class_entry.get_text().c_str());
	if (new_class_name != cur_class_name)
	{
		keys_view.set_model(RefPtr<ListStore>(NULL));

		try
		{
			CIMClass cimClass = CIMOMHandle::getClass(new_class_name);
			props = cimClass.getKeys();

			keys_store = ListStore::create(KeysColumns);
			for (unsigned int i = 0; i < props.size(); i++)
			{
				TreeIter iter = keys_store->append();
				TreeRow row = *iter;
				row[KeysColumns.name] = ustring(props[i].getName().c_str());
				row[KeysColumns.value] = "";
				row[KeysColumns.propindex] = i;
			}
			keys_view.set_model(keys_store);
		}
		catch (...)
		{
		}
		
		cur_class_name = new_class_name;
	}
}

CIMObjectPath
AddCIMObjectDialog::get_object_path() const
{
	CIMObjectPath result(cur_class_name, props);
	result.setNameSpace(CIMOMHandle::getDefaultNamespace());
	return result;
}

void
AddCIMObjectDialog::on_value_edited(const ustring &path,
		const ustring &new_text)
{
	assert(keys_store);

	TreeIter iter = keys_store->get_iter(path);
	TreeRow row = *iter;

	CIMProperty *prop = &props[row[KeysColumns.propindex]];
	switch (prop->getDataType().getType())
	{
		case CIMDataType::STRING:
		{
			String str(new_text.c_str());
			prop->setValue(CIMValue(str));
			break;
		}
		default:
			g_warning("Unhandled data type");
			return;
	}

	row[KeysColumns.value] = new_text;	
}
