// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "BoundEntry.h"
#include <openwbem/OW_CIMProperty.hpp>
#include <openwbem/OW_CIMValue.hpp>
#include <openwbem/OW_CIMQualifier.hpp>
#include <iostream>

using namespace OpenWBEM;
using namespace Gtk;

BoundEntry::BoundEntry(const String &property_name)
{
	this->property_name = property_name;
	this->entry = NULL;
	this->label = NULL;
}

BoundEntry::~BoundEntry()
{
	if (entry != NULL)
		delete entry;
	if (label != NULL)
		delete label;
}

void
BoundEntry::create_entry()
{
	entry = new Entry();
	entry->show();
}

void
BoundEntry::create_label()
{
	label = new Label();
	label->set_alignment(ALIGN_LEFT, ALIGN_CENTER);
	label->show();
}

Glib::ustring
BoundEntry::get_property_display_name()
{
	CIMProperty prop = objectref->getProperty(property_name);
	CIMQualifier qualifier = prop.getQualifier("DisplayName");
	if (qualifier != NULL)
	{
		CIMValue value = qualifier.getValue();
		if (value)
		{
			String str;
			value.get(str);
			return str.c_str();
		}
	}
	return property_name.c_str();
}

void
BoundEntry::update()
{
	CIMProperty prop = objectref->getProperty(property_name);
	CIMValue value = prop.getValue();
	if (value)
	{
		entry->set_text(value.toString().c_str());
	}
	else
	{
		entry->set_text("");
	}
}

void
BoundEntry::set_inst(const CIMInstanceHandle &objectref)
{
	this->objectref = objectref;
	if (label != NULL)
	{
		label->set_text(get_property_display_name() + ":");
	}
}

void
BoundEntry::applyChanges()
{
	CIMProperty prop = objectref->getProperty(property_name);
	switch (prop.getDataType().getType())
	{
	case CIMDataType::STRING:
		objectref->setProperty(property_name, CIMValue(entry->get_text().c_str()));
		break;

	case CIMDataType::UINT32:
		UInt32 i(atoi(entry->get_text().c_str()));
		objectref->setProperty(property_name, CIMValue(i));
		break;
	}
}
