// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "CIMInstanceProxy.h"
#include "CIMOMHandle.h"
#include <iostream>
#include <openwbem/OW_CIMProperty.hpp>
#include <openwbem/OW_CIMValue.hpp>
#include <openwbem/OW_CIMException.hpp>

using namespace OpenWBEM;

class InstanceCreator : public CIMInstanceProxy
{
private:
	bool created;

public:
	InstanceCreator(const String &ns, const CIMInstance &inst)
		: CIMInstanceProxy(ns, inst)
	{
		created = false;
	}
	
	virtual void commitChanges();
};

CIMInstanceProxy::CIMInstanceProxy(const CIMObjectPath &cop)
	: cop(cop)
{
	this->fetched_instance = false;
	
//	std::cout << "Created CIMInstanceProxy!" << std::endl;
//	std::cout << cop.toString() << std::endl;
}

CIMInstanceProxy::CIMInstanceProxy(const String &ns, const CIMInstance &inst)
	: inst(inst)
{
	this->cop = CIMObjectPath(ns, inst);
	this->fetched_instance = true;
}

CIMInstanceProxy::~CIMInstanceProxy()
{
//	std::cout << "Deleted CIMObjectHandle!" << std::endl;
}

CIMInstanceHandle
CIMInstanceProxy::create(const CIMObjectPath &cop)
{
	return CIMInstanceHandle(new CIMInstanceProxy(cop));
}

CIMInstanceHandle
CIMInstanceProxy::new_instance(const String &ns, const String &className)
{
	CIMClass cimClass = CIMOMHandle::getClass(className);
	CIMInstance inst = cimClass.newInstance();
	return CIMInstanceHandle(new InstanceCreator(ns, inst));
}

void
CIMInstanceProxy::fetch_instance()
{
	if (fetched_instance)
		return;

	this->inst = CIMOMHandle::getInstance(cop);
	this->fetched_instance = true;
}

void
CIMInstanceProxy::commitChanges()
{
	fetch_instance();
	
	// TODO: check to see if the instance data even got modified
	CIMOMHandle::modifyInstance(inst);
}

void
CIMInstanceProxy::destroy()
{
	CIMOMHandle::deleteInstance(cop);
}

CIMPropertyArray
CIMInstanceProxy::getProperties()
{
	fetch_instance();
	return inst.getProperties();
}

CIMProperty
CIMInstanceProxy::getProperty(const String &propertyName)
{
	fetch_instance();
	return inst.getProperty(propertyName);
}

CIMInstanceHandle
CIMInstanceProxy::getRefProperty(const String &propertyName)
{
	CIMProperty prop = getProperty(propertyName);
	CIMObjectPath cop = prop.getValue().toCIMObjectPath();
	return create(cop);
}

void
CIMInstanceProxy::setProperty(const String &propertyName,
		const CIMValue &value)
{
	fetch_instance();
	inst.setProperty(propertyName, value);
}

void
CIMInstanceProxy::setRefProperty(const String &propertyName,
	CIMInstanceHandle value)
{
	setProperty(propertyName, CIMValue(value->getObjectPath()));
}

std::vector<CIMInstanceHandle>
CIMInstanceProxy::references(const String &resultClass)
{
	CIMObjectPathEnumeration copEnum = CIMOMHandle::referenceNamesE(cop,
		resultClass);
	std::vector<CIMInstanceHandle> result;
	
	while (copEnum.hasMoreElements())
	{
		CIMObjectPath cop = copEnum.nextElement();
		CIMInstanceHandle handle = create(cop);
		result.push_back(handle);
	}
	return result;
}

String
CIMInstanceProxy::toString() const
{
	return cop.toString();
}

void
InstanceCreator::commitChanges()
{
	if (created)
	{
		// superclass method
		CIMInstanceProxy::commitChanges();
	}
	else
	{
		CIMOMHandle::createInstance(inst);
		cop = CIMObjectPath(cop.getNameSpace(), inst);
		created = true;
	}
}
