// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef CLASSINFORMATIONDIALOG_H
#define CLASSINFORMATIONDIALOG_H

#include <gtkmm/dialog.h>
#include <gtkmm/scrolledwindow.h>
#include <gtkmm/textview.h>
#include <gtkmm/treeview.h>
#include <gtkmm/liststore.h>
#include <gtkmm/notebook.h>
#include <openwbem/OW_CIMClass.hpp>
#include <openwbem/OW_CIMProperty.hpp>
#include "AssociatorClassList.h"

/**
 * A dialog to display schema information for a CIM class.
 */
class ClassInformationDialog : public Gtk::Dialog
{
private:
	// PROPERTIES
	OpenWBEM::CIMClass klass;

	// WIDGETS
	Gtk::Notebook notebook;

	Gtk::VBox description_vbox;
	Gtk::Label superclass_label;
	Gtk::ScrolledWindow description_scrolled;
	Gtk::TextView description_view;

	Gtk::ScrolledWindow properties_scrolled;
	Gtk::TreeView properties_view;
	Glib::RefPtr<Gtk::ListStore> properties_store;

	Gtk::ScrolledWindow associations_scrolled;
	AssociatorClassList associations_list;

	Gtk::ScrolledWindow subclasses_scrolled;
	ClassList subclasses_list;

public:
	/**
	 * Create a class information dialog.
	 */
	ClassInformationDialog(OpenWBEM::CIMClass klass);

protected:
	// SIGNAL HANDLERS
	void property_row_activated(const Gtk::TreeModel::Path &path,
			Gtk::TreeViewColumn *column);
	virtual void on_response(int response_id);

private:
	class PropertyColumnsModel : public Gtk::TreeModel::ColumnRecord
	{
	public:
		PropertyColumnsModel()
		{
			add(name);
			add(datatype);
			add(propagated);
			add(key);
			add(property);
		}
		Gtk::TreeModelColumn<Glib::ustring> name;
		Gtk::TreeModelColumn<Glib::ustring> datatype;
		Gtk::TreeModelColumn<bool> propagated;
		Gtk::TreeModelColumn<bool> key;
		Gtk::TreeModelColumn<OpenWBEM::CIMProperty> property;
	};
	
	PropertyColumnsModel PropertyColumns;

	void list_properties();
};

#endif /* CLASSINFORMATIONDIALOG_H */
