// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "ConnectDialog.h"
#include <gtkmm/stock.h>

using namespace Gtk;
using namespace OpenWBEM;

ConnectDialog::ConnectDialog()
	: table(4, 2),
	url_label("URL:"),
	namespace_label("Namespace:"),
	username_label("Username:"),
	password_label("Password:")
{
	set_title("Connect to WBEM Server");
	set_default_size(300, 200);

	url_label.set_alignment(ALIGN_LEFT, ALIGN_CENTER);
	namespace_label.set_alignment(ALIGN_LEFT, ALIGN_CENTER);
	username_label.set_alignment(ALIGN_LEFT, ALIGN_CENTER);
	password_label.set_alignment(ALIGN_LEFT, ALIGN_CENTER);

	// password should not be visible
	password_entry.set_visibility(false);

	// initial values
	url_entry.set_text("http://localhost/cimom");
	namespace_entry.set_text("root/cimv2");

	// allow users to OK dialog by pressing Enter
	url_entry.set_activates_default(true);
	namespace_entry.set_activates_default(true);
	username_entry.set_activates_default(true);
	password_entry.set_activates_default(true);
	
	// add widget table to this dialog
	table.set_row_spacings(8);
	table.set_col_spacings(10);
	table.set_border_width(8);
	get_vbox()->pack_start(table);

	// add labels and entries to table
	table.attach(url_label, 0, 1, 0, 1, FILL);
	table.attach(namespace_label, 0, 1, 1, 2, FILL);
	table.attach(username_label, 0, 1, 2, 3, FILL);
	table.attach(password_label, 0, 1, 3, 4, FILL);

	table.attach(url_entry, 1, 2, 0, 1);
	table.attach(namespace_entry, 1, 2, 1, 2);
	table.attach(username_entry, 1, 2, 2, 3);
	table.attach(password_entry, 1, 2, 3, 4);

	// add buttons
	add_button(Stock::CANCEL, RESPONSE_CANCEL);
	add_button(Stock::OK, RESPONSE_OK);
	set_default_response(RESPONSE_OK);
	
	// make sure all children widgets are visible
	show_all_children();
}

void
ConnectDialog::getParams(OpenWBEM::String &url, OpenWBEM::String &ns,
		OpenWBEM::String &username, OpenWBEM::String &password)
{
	url = url_entry.get_text().c_str();
	ns = namespace_entry.get_text().c_str();
	username = username_entry.get_text().c_str();
	password = password_entry.get_text().c_str();
}
