// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "DefaultInstanceScopeItem.h"
#include "MainWindow.h"
#include "ClassHandlerRegistry.h"
#include "CIMOMHandle.h"
#include "ModifyInstanceDialog.h"
#include "InstancePropertyListForm.h"
#include "InstanceAssociationsForm.h"
#include "PropertyPagesDialog.h"
#include <openwbem/OW_CIMValue.hpp>
#include <openwbem/OW_CIMProperty.hpp>
#include <openwbem/OW_CIMException.hpp>
#include <gtkmm/stock.h>

using namespace OpenWBEM;
using namespace Glib;
using namespace Gtk;

DefaultInstanceScopeItem::DefaultInstanceScopeItem(const CIMObjectPath &cop,
	MainWindow *main_window)
{
	this->cop = cop;
	this->objectref = CIMInstanceProxy::create(cop);
	this->main_window = main_window;
}

void
DefaultInstanceScopeItem::activate()
{
	String assocClassName = get_child_assoc_name();
	if (assocClassName != "")
	{
		// display children in result pane
		main_window->get_result_list()->remove_all_columns();
		main_window->get_result_list()->add_text_column("ElementName", "Name");
		main_window->get_result_list()->associators(cop, assocClassName);
	}
	else
	{
		ClassHandler *handler = ClassHandlerRegistry::get_handler(
			cop.getClassName());
		assert(handler != NULL);

		handler->activate(cop, main_window);
	}
}

String
DefaultInstanceScopeItem::get_child_assoc_name()
{
	ClassHandler *handler = ClassHandlerRegistry::get_handler(
			cop.getClassName());
	assert(handler != NULL);

	return handler->get_child_assoc_name();
}

void
DefaultInstanceScopeItem::expand_row(TreeModel::Row &row)
{
	String assocClassName = get_child_assoc_name();
	if (assocClassName != "")
	{
		try
		{
			CIMInstanceEnumeration result;
			result = CIMOMHandle::associatorsE(cop, assocClassName);

			while (result.hasMoreElements())
			{
				CIMInstance inst = result.nextElement();
				CIMObjectPath cop(CIMOMHandle::getDefaultNamespace(), inst);
				CIMProperty prop = inst.getProperty("ElementName");
				if (prop)
				{
					CIMValue value = prop.getValue();
					if (value)
					{
						String str;
						value.get(str);
				
						ClassHandler *handler
							= ClassHandlerRegistry::get_handler(
								inst.getClassName());
						assert(handler != NULL);

						main_window->add_scope_item(
								row, str.c_str(),
								handler->create_scope_item(cop, main_window));
					}
				}
			} // while has more elements
		}
		catch (CIMException e)
		{
			g_warning("%s", e.getMessage());
		}
	}
}

void
DefaultInstanceScopeItem::init_actions_menu(Menu &action_menu)
{
	Menu::MenuList &menulist = action_menu.items();
	menulist.push_back(Menu_Helpers::StockMenuElem(Stock::PROPERTIES,
		SigC::slot(*this, &DefaultInstanceScopeItem::on_menu_properties)));
}

void
DefaultInstanceScopeItem::on_menu_properties()
{
	PropertyPagesDialog *dlg = new PropertyPagesDialog;
	dlg->set_title("Properties");
	dlg->set_transient_for(*main_window);
	dlg->add_form(new InstancePropertyListForm(objectref), "Properties");
	try
	{
		dlg->add_form(new InstanceAssociationsForm(objectref), "Associations");
	}
	catch (const CIMException &e)
	{
		g_warning("Couldn't add associations form: %s",
			e.getMessage());
	}
	dlg->show();
}
