// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef DEFAULTINSTANCESCOPEITEM_H
#define DEFAULTINSTANCESCOPEITEM_H

#include "ScopeItem.h"
#include "CIMInstanceProxy.h"

/**
 * Implements a scope item that represents a specific CIM instance.
 * Create it by identifying a CIM instance by its path (CIMObjectPath).
 * It will identify the class of the object and use the class's
 * "ClassHandler" to implement the behaviors of the scope item.
 */
class DefaultInstanceScopeItem : public ScopeItem,
	  public SigC::Object
{
protected:
	void on_menu_properties();

protected:
	/**
	 * The CIM object path of the instance that this scope item
	 * represents.
	 */
	OpenWBEM::CIMObjectPath cop;
	CIMInstanceHandle objectref;

	/**
	 * Pointer to the main window that owns this scope item.
	 */
	MainWindow *main_window;

	virtual OpenWBEM::String get_child_assoc_name();
	/**
	 * Adds an action menu item for viewing properties of the item.
	 */
	virtual void init_actions_menu(Gtk::Menu &action_menu);

public:
	/**
	 * Create a new scope item for the specified CIMObjectPath and
	 * main window.
	 */
	DefaultInstanceScopeItem(const OpenWBEM::CIMObjectPath &cop,
		MainWindow *main_window);
	virtual void activate();
	virtual void expand_row(Gtk::TreeModel::Row &row);
};

#endif /* DEFAULTINSTANCESCOPEITEM_H */
