// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef INSTANCEPROPERTYLIST_H
#define INSTANCEPROPERTYLIST_H

#include <gtkmm/treeview.h>
#include <gtkmm/liststore.h>
#include <openwbem/OW_CIMObjectPath.hpp>
#include <openwbem/OW_CIMInstance.hpp>
#include <openwbem/OW_CIMProperty.hpp>
#include <openwbem/OW_Reference.hpp>
#include "CIMInstanceProxy.h"

/**
 * A list box displaying an instance's properties and their values.
 * Double-click a property to edit its value. You can also right-click
 * which generates a popup menu with the options of "Edit" and "Clear."
 * "Clear" sets the selected property value to NULL.
 */
class InstancePropertyList : public Gtk::TreeView
{
private:
	// PROPERTIES
	CIMInstanceHandle instHandle;

	Glib::RefPtr<Gtk::ListStore> store;
	bool dirty;

public:
	// CONSTRUCTOR
	InstancePropertyList();
	InstancePropertyList(OpenWBEM::CIMObjectPath cop);
	InstancePropertyList(OpenWBEM::CIMObjectPath cop,
		OpenWBEM::Reference<OpenWBEM::CIMInstance> inst);
	InstancePropertyList(CIMInstanceHandle instHandle);
	~InstancePropertyList();

	void list_properties(CIMInstanceHandle instHandle);

	// MUTATORS
	void applyChanges();

	// ACCESSORS
	bool isDirty() const
		{ return dirty; }

protected:
	// SIGNAL HANDLERS
	/// Called whenever a row is double-clicked.
	virtual void on_row_activated(const Gtk::TreeModel::Path &path,
			Gtk::TreeViewColumn *column);
	/** Called whenever a mouse-button is pressed.
	 * This handler generates a popup-menu if the right mouse button
	 * is clicked.
	 */
	virtual bool on_button_press_event(GdkEventButton *event);
	/// Called when "Edit" is selected from the context menu.
	void on_menu_edit();
	/// Called when "Clear" is selected from the context menu.
	void on_menu_clear();

private:
	class PropertyColumnsModel : public Gtk::TreeModel::ColumnRecord
	{
	public:
		PropertyColumnsModel()
		{
			add(name);
			add(datatype);
			add(value);
			add(property);
		}
		Gtk::TreeModelColumn<Glib::ustring> name;
		Gtk::TreeModelColumn<Glib::ustring> datatype;
		Gtk::TreeModelColumn<Glib::ustring> value;
		Gtk::TreeModelColumn<OpenWBEM::CIMProperty> property;
	};
	
	PropertyColumnsModel PropertyColumns;
	
	void initialize();
	void setRowData(Gtk::TreeModel::Row &row, const OpenWBEM::CIMProperty &prop);
};

#endif /* INSTANCEPROPERTYLIST_H */
