// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef INSTANCEPROPERTYLISTFORM_H
#define INSTANCEPROPERTYLISTFORM_H

#include <gtkmm/scrolledwindow.h>

#include <openwbem/OW_CIMObjectPath.hpp>
#include <openwbem/OW_CIMInstance.hpp>
#include <openwbem/OW_CIMProperty.hpp>
#include "Form.h"
#include "InstancePropertyList.h"
#include "CIMInstanceProxy.h"

/**
 * A form for editing all the properties of a CIM instance. This form
 * presents a scrollable list of all the properties that make up the
 * instance and their values, and provides a way to edit these
 * properties. Editing the individual property values is done with the
 * EditPropertyDialog.
 */
class InstancePropertyListForm : public Form
{
private:
	/**
	 * A handle to the CIM instance this form is representing.
	 */
	OpenWBEM::Reference<OpenWBEM::CIMInstance> inst;

	InstancePropertyList properties_list;
	Gtk::ScrolledWindow properties_scrolled;

public:
	// CONSTRUCTOR
	InstancePropertyListForm();
	InstancePropertyListForm(CIMInstanceHandle instHandle);

	// METHODS DEFINED IN Form
	virtual void set_instance(const OpenWBEM::CIMObjectPath &cop,
			OpenWBEM::Reference<OpenWBEM::CIMInstance> inst);
	virtual Gtk::Widget *getWidget();
	virtual void applyChanges();
};

#endif /* INSTANCEPROPERTYLISTFORM_H */
