// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

// Linux System plugin headers
#include "LinuxSystemPlugin.h"

// CIMBrowser headers
#include "ClassHandlerRegistry.h"
#include "ModifyInstanceDialog.h"
#include "InstancePropertyListForm.h"
#include "CIMOMHandle.h"

// Gtkmm headers
#include <gtkmm/messagedialog.h>

// OpenWBEM headers
#include <openwbem/OW_CIMException.hpp>
#include <openwbem/OW_CIMValue.hpp>

// System headers
#include <assert.h>

using namespace OpenWBEM;
using namespace Gtk;

/**
 * A scope item representing a CIM instance of
 * Config4GNU_LinuxComputerSystem.
 * @ingroup LinuxSystemPlugin
 * This scope item adds menu items for configuring a Linux computer
 * system and provides implementations for those menu items.
 */
class LinuxComputerSystemScopeItem
	: public DefaultInstanceScopeItem
{
public:
	/**
	 * Creates a new scope item representing a Linux Computer System.
	 */
	LinuxComputerSystemScopeItem(const CIMObjectPath &cop,
			MainWindow *main_window)
		: DefaultInstanceScopeItem(cop, main_window)
	{
	}

	virtual void expand_row(TreeModel::Row &row);

private:
	void do_global_settings();
	void do_default_share_settings();
	void do_new_share();
	void do_delete_share();

	void refresh()
	{
		activate();
	}
};

class LinuxSystemServicesScopeItem
	: public DefaultInstanceScopeItem
{
public:
	LinuxSystemServicesScopeItem(const CIMObjectPath &cop,
			MainWindow *main_window)
		: DefaultInstanceScopeItem(cop, main_window)
	{
	}
	virtual String get_child_assoc_name();
};

class LinuxComputerSystemHandler : public ClassHandler
{
public:
	virtual void do_properties(const CIMObjectPath &cop, Window *parent_window)
	{
	}

	virtual void activate(const CIMObjectPath &cop, MainWindow *main_window)
	{
	}
	
	virtual ScopeItemRef create_scope_item(const CIMObjectPath &cop,
		MainWindow *main_window)
	{
		return ScopeItemRef(new LinuxComputerSystemScopeItem(cop, main_window));
	}
};

void
LinuxSystemPlugin::init()
{		
	ClassHandlerRegistry::add_handler("Config4GNU_LinuxComputerSystem",
		new LinuxComputerSystemHandler);
}

void
LinuxComputerSystemScopeItem::expand_row(TreeModel::Row &row)
{
	main_window->add_scope_item(
			row, "Services",
			ScopeItemRef(new LinuxSystemServicesScopeItem(cop, main_window)));
}

void
LinuxComputerSystemScopeItem::do_global_settings()
{
}

void
LinuxComputerSystemScopeItem::do_default_share_settings()
{
}

void
LinuxComputerSystemScopeItem::do_new_share()
{
}

void
LinuxComputerSystemScopeItem::do_delete_share()
{
}

String
LinuxSystemServicesScopeItem::get_child_assoc_name()
{
	return "CIM_HostedService";
}
