// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "PropertyPagesDialog.h"

#include <gtkmm/messagedialog.h>
#include <gtkmm/stock.h>

using Glib::ustring;
using namespace Gtk;
using namespace OpenWBEM;

PropertyPagesDialog::PropertyPagesDialog()
{
	set_default_size(300, 300);

	// add notebook to dialog
	notebook.show();
	get_vbox()->pack_start(notebook);

	// add buttons
	add_button(Stock::CANCEL, RESPONSE_CANCEL);
	add_button(Stock::APPLY, RESPONSE_APPLY);
	add_button(Stock::OK, RESPONSE_OK);
	set_default_response(RESPONSE_OK);
}

PropertyPagesDialog::~PropertyPagesDialog()
{
	for (unsigned int i = 0; i < forms.size(); i++)
	{
		delete forms[i];
	}
}

void
PropertyPagesDialog::add_form(Form *form, Glib::ustring title)
{
	forms.push_back(form);
	form->getWidget()->show();
	notebook.append_page(*form->getWidget(), title);
}

void
PropertyPagesDialog::on_response(int response_id)
{
	if (response_id == RESPONSE_APPLY || response_id == RESPONSE_OK)
	{
		try
		{
			applyChanges();
		}
		catch (const Exception &e)
		{
			MessageDialog dialog(*this, e.getMessage(), MESSAGE_ERROR);
			dialog.run();
			return;
		}
	}

	if (response_id == RESPONSE_CANCEL || response_id == RESPONSE_OK)
		delete this;
}

void
PropertyPagesDialog::applyChanges()
{
	for (unsigned int i = 0; i < forms.size(); i++)
	{
		forms[i]->applyChanges();
	}
}
