// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef SIMPLECIMFORM_H
#define SIMPLECIMFORM_H

#include "Form.h"
#include "BoundEntry.h"
#include "CIMInstanceProxy.h"
#include <vector>

/**
 * A base class for forms that represent a single CIM instance.
 * The SimpleCIMForm class handles the creation of BoundEntry widgets
 * and manages them automatically... updating their widgets initially
 * and applying their inputs to the underlying CIM instance.
 *
 * This form uses the new CIMObjectHandle class for manipulating the
 * underlying CIM instance.
 */
class SimpleCIMForm : public Form
{
private:
	/**
	 * A handle to the CIM instance this form is representing.
	 */
	CIMInstanceHandle objectref;

	/**
	 * The widgets on the form.
	 */
	std::vector<BoundEntry *> bound_entries;

protected:
	/**
	 * Create a new SimpleCIMForm.
	 */
	SimpleCIMForm(CIMInstanceHandle objectref);

	/**
	 * Fills in widgets with current values of instance.
	 */
	void update();

public:
	virtual ~SimpleCIMForm();

	/**
	 * Create a new BoundEntry and sets it to be managed by this form.
	 * The caller should not "delete" the returned value, because it is
	 * referenced by the form itself and deleted when the form is
	 * deleted.
	 */
	virtual BoundEntry *new_entry(const OpenWBEM::String &propertyName);

	// IMPLEMENTED METHODS DEFINED IN Form
	virtual void set_instance(const OpenWBEM::CIMObjectPath &cop,
			OpenWBEM::Reference<OpenWBEM::CIMInstance> inst);
	virtual void applyChanges();
};

#endif /* SIMPLECIMFORM_H */
