// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "main.h"
#include "MainWindow.h"
#include "ConnectDialog.h"

// plugin headers
#include "LinuxSystemPlugin.h"

// system headers
#include <gtkmm.h>
#include <ltdl.h>
#include <iostream>

using namespace OpenWBEM;
using namespace Gtk;

static void init_libltdl();
static void load_plugin(const char *plugin_name);

typedef void PluginInitFunc();

int
main(int argc, char *argv[])
{
	Gtk::Main kit(argc, argv);

	// plugin initialization
	init_libltdl();
	load_plugin("dns");
	load_plugin("nagios");
	load_plugin("samba");

	LinuxSystemPlugin::init();

	MainWindow window;
	try
	{
		window.init_scope_list();
	}
	catch (const Exception &e)
	{
		MessageDialog dialog(e.getMessage(), MESSAGE_ERROR);
		dialog.set_title(e.type());
		dialog.run();
		exit(1);
	}

	Gtk::Main::run(window);

	return 0;
}

static void
init_libltdl()
{
	// Initialize libltdl
	int errors = lt_dlinit();
	if (errors)
	{
		std::cerr << "Could not initialize libltdl." << std::endl;
		exit(1);
	}

	// Set the module search path
	errors = lt_dlsetsearchpath(PLUGINSDIR);
	if (errors)
	{
		std::cerr << "Could not set search path." << std::endl;
		exit(1);
	}
}

static void
load_plugin(const char *plugin_name)
{

	// Load the module
	lt_dlhandle module = lt_dlopenext(plugin_name);
	if (!module)
	{
		std::cerr << "Error loading plugin: " << plugin_name << std::endl;
		std::cerr << lt_dlerror() << std::endl;
		return;
	}

	PluginInitFunc *init = (PluginInitFunc *) lt_dlsym(module, "init_plugin");

	// check for errors
	if (init == NULL)
	{
		std::cerr << "Could not find init_plugin in plugin: "
			<< plugin_name << std::endl;
		std::cerr << lt_dlerror() << std::endl;
		lt_dlclose(module);
		return;
	}

	// call the entry point function
	(*init)();

	std::cout << "Plugin loaded: " << plugin_name << std::endl;
}
