// Copyright 2004 Jason Long

// This file is part of cimbrowser.
//
// Cimbrowser is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Cimbrowser is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Cimbrowser; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#ifndef MAIN_H
#define MAIN_H

int main(int argc, char *argv[]);

/** @mainpage
 *
 * @section intro Introduction
 *
 * CIMBrowser is a WBEM client that specializes in configuration
 * aspects. It aims to resemble functionality found in the Microsoft
 * Management Console, developed by Microsoft Corporation. It can
 * list CIM instances, edit properties, traverse associations, and
 * browse the CIM schema.
 *
 * @section mainwindow The main window
 *
 * CIMBrowser's main window is implemented in the MainWindow class.
 * The window is a split-pane window. The left pane is called the
 * "scope" pane and the right pane is called the "result" pane. The
 * scope pane is a tree of named scope items. Selecting a scope item
 * will display something in the result pane, usually a list of CIM
 * instances.
 *
 * Items found in the scope pane extend the ScopeItem class. This class
 * contains methods that define what happens when a scope item is
 * selected, what menu items should be added to the Action menu, and
 * if the scope item has subitems.
 *
 * @section propertiesdialog The modify instance dialog
 *
 * When a result item is double-clicked, the typical action is to
 * display a dialog letting you edit the properties of that item. This
 * is implemented in the ModifyInstanceDialog. The ModifyInstanceDialog
 * contains a notebook with one or more tabs. Each tab displays a
 * Form object, which facilitates the manipulation of a CIM instance.
 *
 * Plugins provide implementation of Form objects, but some work has
 * been done to simplify this implementation. This work consists of the
 * SimpleCIMForm class, which handles connecting the form to a
 * particular CIM instance and allows the form to contain BoundEntry
 * objects. These BoundEntry objects automatically provide an
 * appropriate widget for editing a property value and a label that
 * displays the descriptive text for the property found in the CIM
 * schema. 
 *
 * @section plugins Plugins
 *
 * Currently plugins are not really plugins, since they are statically
 * linked into the CIMBrowser executable. However, they are designed so
 * that they can eventually be separate modules. The plugins contain
 * an init function which registers class handlers for one or more
 * CIM class names. Once registered, the CIMBrowser will allow the
 * plugin to handle activation of and displaying properties for
 * instances of those classes. 
 *
 * The CIMBrowser project currently contains plugins for DNS,
 * Nagios, and Samba.
 */

#endif /* MAIN_H */
